/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    userdefine.h
* @brief   This file includes user definition.
* @version 1.0.0
* @date    2022/2/25
***********************************************************************************************************************/
#ifndef _USER_DEF_H
#define _USER_DEF_H

/***********************************************************************************************************************
User definitions
***********************************************************************************************************************/
#ifndef __TYPEDEF__

typedef unsigned short 	MD_STATUS;
/* Status list definition */
#define MD_STATUSBASE        (0x00U)
#define MD_OK                (MD_STATUSBASE + 0x00U) /* register setting OK */
#define MD_SPT               (MD_STATUSBASE + 0x01U) /* IIC stop */
#define MD_NACK              (MD_STATUSBASE + 0x02U) /* IIC no ACK */
#define MD_BUSY1             (MD_STATUSBASE + 0x03U) /* busy 1 */
#define MD_BUSY2             (MD_STATUSBASE + 0x04U) /* busy 2 */
#define MD_OVERRUN           (MD_STATUSBASE + 0x05U) /* IIC OVERRUN occur */

/* Error list definition */
#define MD_ERRORBASE         (0x80U)
#define MD_ERROR             (MD_ERRORBASE + 0x00U)  /* error */
#define MD_ARGERROR          (MD_ERRORBASE + 0x01U)  /* error agrument input error */
#define MD_ERROR1            (MD_ERRORBASE + 0x02U)  /* error 1 */
#define MD_ERROR2            (MD_ERRORBASE + 0x03U)  /* error 2 */
#define MD_ERROR3            (MD_ERRORBASE + 0x04U)  /* error 3 */
#define MD_ERROR4            (MD_ERRORBASE + 0x05U)  /* error 4 */
#define MD_ERROR5            (MD_ERRORBASE + 0x06U)  /* error 5 */
#endif

#define TRMW   *((volatile uint8_t *)(0x40021C08))
#define TRMR   *((volatile uint8_t *)(0x40021C09))
#define TRMT   *((volatile uint8_t *)(0x40021C0A))

/***********************************************************************************************************************
Macro definitions for Clock
***********************************************************************************************************************/
//#define CPU_CLOCK_FREQ 12000000UL//12MHz clock frequency



/***********************************************************************************************************************
Macro definitions of SCI usage:
Each channel of SCI has three functions: UART, SPI, and IIC. You can only choose one function to use.
***********************************************************************************************************************/
/* ToDo: You can only define ONE of the following THREE MACROs according to your application */
#define USE_SCI_UART0_TX      /*! Using CH0 of SCI0 as UART Transmitter */
#define USE_SCI_SPI00         /*! Using CH0 of SCI0 as SPI Transmitter or Receiver */
#define USE_SCI_IIC00         /*! Using CH0 of SCI0 as IIC Transmitter or Receiver */

/* ToDo: You can only define ONE of the following THREE MACROs according to your application */
#define USE_SCI_UART0_RX      /*! Using CH1 of SCI0 as UART Receiver */

/* ToDo: You can only define ONE of the following THREE MACROs according to your application */
#define USE_SCI_UART2_TX      /*! Using CH0 of SCI1 as UART Transmitter */
#define USE_SCI_SPI20         /*! Using CH0 of SCI1 as SPI Transmitter or Receiver */
#define USE_SCI_IIC20         /*! Using CH0 of SCI1 as IIC Transmitter or Receiver */

/* ToDo: You can only define ONE of the following THREE MACROs according to your application */
#define USE_SCI_UART2_RX      /*! Using CH1 of SCI1 as UART Receiver */

/* ToDo: You can define the following MACRO and USE_SCI_UART2_TX or USE_SCI_UART2_RX MACROs to use Infra-Red transmitter or receiver */
#define USE_IRDA              /*! Enable Infra-Red transmission with UART2 */

/***********************************************************************************************************************
Macro definitions of LIN usage:
***********************************************************************************************************************/
#define LIN_MASTER
//#define LIN_SLAVE

/***********************************************************************************************************************
DMA Control Data Set definitions
***********************************************************************************************************************/
#define SPI00_WITH_DMA
#define CTRL_DATA_SPI00 0

#define SPI20_WITH_DMA
#define CTRL_DATA_SPI20 8

#define SPIHS0_WITH_DMA
#define CTRL_DATA_SPIHS0   10

#define ADC_WITH_DMA
#define CTRL_DATA_ADC   12

#define DAC_WITH_DMA
#define CTRL_DATA_DAC   14

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/

typedef enum 
{
    SPI_MODE_0 = 0,  // Mode 0: CPOL = 0, CPHA = 0; i.e. CKP = 1, DAP = 1 
    SPI_MODE_1 = 1,  // Mode 1: CPOL = 0, CPHA = 1; i.e. CKP = 1, DAP = 0 
    SPI_MODE_2 = 2,  // Mode 2: CPOL = 1, CPHA = 0; i.e. CKP = 0, DAP = 1 
    SPI_MODE_3 = 3,  // Mode 3: CPOL = 1, CPHA = 1; i.e. CKP = 0, DAP = 0 
} spi_mode_t;

/** @addtogroup Peripherals_Port_Setting_Definations
  * @{
  */

/* ================================================================================================================== */
/* ================                                       TM40                                       ================ */
/* ================================================================================================================== */
/**
  * @brief TM40 TI00~3 and TO00~3 Port Setting (Alternative to fixed ports)
  */

#define TI00_PORT_SETTING() do{ \
        PORT->PM0  |=  (1 << 0);    /* P00 is used as TI00 input */ \
        PORT->PMC0 &= ~(1 << 0);    /* P00 is digital function */ \
}while(0)

#define TI01_PORT_SETTING() do{ \
        PORT->PM1  |=  (1 << 6);    /* P16 is used as TI01 input */ \
}while(0)

#define TI02_PORT_SETTING() do{ \
        PORT->PM1  |=  (1 << 7);    /* P17 is used as TI02 input */ \
}while(0)

#define TI03_PORT_SETTING() do{ \
        PORT->PM1  |=  (1 << 3);    /* P13 is used as TI03 input */ \
}while(0)

#define TO00_PORT_SETTING() do{ \
        PORT->PCLR0 =  (1 << 1);    /* P01 output low level */ \
        PORT->PM0  &= ~(1 << 1);    /* P01 is used as TO00 output */ \
        PORT->PMC0 &= ~(1 << 1);    /* P01 is digital function */ \
}while(0)

#define TO01_PORT_SETTING() do{ \
        PORT->PCLR1 =  (1 << 6);    /* P16 output low level */ \
        PORT->PM1  &= ~(1 << 6);    /* P16 is used as TO01 output */ \
}while(0)

#define TO02_PORT_SETTING() do{ \
        PORT->PCLR1 =  (1 << 7);    /* P17 output low level */ \
        PORT->PM1  &= ~(1 << 7);    /* P17 is used as TO02 output */ \
}while(0)

#define TO03_PORT_SETTING() do{ \
        PORT->PCLR1 =  (1 << 3);    /* P13 output low level */ \
        PORT->PM1  &= ~(1 << 3);    /* P13 is used as TO03 output */ \
}while(0)
/* ================================================================================================================== */
/* ================                                      TMA                                         ================ */
/* ================================================================================================================== */

/**
  * @brief TAIO Port Setting (Alternative to 4 group ports)
  */

/* ToDo: You can allocate the TAIO to P01, P31, P41, or P06 with PIOR11 and PIOR10 register */
#define TAIO_PORT_SETTING() do{ \
        PORT->PIOR1 |=  (0 << 0);    /* allocate TAIO to P01 */ \
        PORT->PCLR0  =  (1 << 1);    /* P01 output low level */ \
        PORT->PM0   &= ~(1 << 1);    /* P01 is used as TAIO output */ \
        PORT->PMC0  &= ~(1 << 1);    /* P01 is digital function */ \
        PORT->POM0  &= ~(1 << 1);    /* P01 is push-pull output mode */ \
}while(0)

/* ToDo: You can allocate the TAIO to P01, P31, P41, or P06 with PIOR11 and PIOR10 register */ 
#define TAI_PORT_SETTING() do{ \
        PORT->PIOR1 |=  (0 << 0);    /* allocate TAIO to P01 */ \
        PORT->PM0   |=  (1 << 1);    /* P01 is used as TAIO input */ \
}while(0)

/* ToDo: You can allocate the TAO to P30, P50, or P00 with PIOR13 and PIOR12 register */
#define TAO_PORT_SETTING() do{ \
        PORT->PIOR1 |=  (1 << 2);    /* allocate TAO to P50 */ \
        PORT->PCLR5 =  (1 << 0);     /* P50 output low level */ \
        PORT->PM5   &= ~(1 << 0);    /* P50 is used as TAO output */ \
        PORT->POM5  &= ~(1 << 0);    /* P50 is push-pull output mode */ \
}while(0)

/* ================================================================================================================== */
/* ================                                      TMB                                         ================ */
/* ================================================================================================================== */

/**
  * @brief TMB Port Setting (Alternative to fixed port)
  */

#define TBCLK0_PORT_SETTING() do{ \
        PORT->PM0   |=  (1 << 0);    /* P00 is used as TBCLK0 input */ \
}while(0)

#define TBCLK1_PORT_SETTING() do{ \
        PORT->PM0   |=  (1 << 1);    /* P01 is used as TBCLK1 input */ \
}while(0)

#define TBI0_PORT_SETTING() do{ \
        PORT->PM5   |=  (1 << 0);    /* P50 is used as TBIO0 input */ \
}while(0)

#define TBI1_PORT_SETTING() do{ \
        PORT->PM5   |=  (1 << 1);    /* P51 is used as TBIO1 input */ \
}while(0)

#define TBO0_PORT_SETTING() do{ \
        PORT->PCLR5  =  (1 << 0);    /* P50 output low level */ \
        PORT->PM5   &= ~(1 << 0);    /* P50 is used as TBIO0 output */ \
        PORT->POM5  &= ~(1 << 0);    /* P50 is push-pull output mode */ \
}while(0)

#define TBO1_PORT_SETTING() do{ \
        PORT->PCLR5  =  (1 << 1);    /* P51 output low level */ \
        PORT->PM5   &= ~(1 << 1);    /* P51 is used as TBIO1 output */ \
        PORT->POM5  &= ~(1 << 1);    /* P51 is push-pull output mode */ \
}while(0)

/* ================================================================================================================== */
/* ================                                      TMM                                         ================ */
/* ================================================================================================================== */

/**
  * @brief TMM Port Setting (Alternative to 3 group ports)
  */

//#define TMCLK_PORT_SETTING() do{ \
//        PORT->PM1   |=  (1 << 7);    /* P17 is used as TMCLK input */ \
//}while(0)

//#define TMIA0_PORT_SETTING() do{ \
//        PORT->PM1   |=  (1 << 7);    /* P17 is used as TMIOA0 input */ \
//}while(0)

/* ToDo: You can allocate the TMIOB0 to P14, P12, or P15 with PIOR37 and PIOR36 register */
#define TMIB0_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 4);    /* P14 is used as TMIOB0 input */ \
}while(0)

#define TMIC0_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 3);    /* P13 is used as TMIOC0 input */ \
}while(0)

/* ToDo: You can allocate the TMIOD0 to P15, P15, or P14 with PIOR37 and PIOR36 register */
#define TMID0_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 5);    /* P15 is used as TMIOD0 input */ \
}while(0)

/* ToDo: You can allocate the TMIOA1 to P12, P11, or P13 with PIOR37 and PIOR36 register */
#define TMIA1_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 2);    /* P12 is used as TMIOA1 input */ \
}while(0)

/* ToDo: You can allocate the TMIOB1 to P10, P10, or P12 with PIOR37 and PIOR36 register */
#define TMIB1_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 0);    /* P10 is used as TMIOB1 input */ \
}while(0)

/* ToDo: You can allocate the TMIOC1 to P13, P14, or P11 with PIOR37 and PIOR36 register */
#define TMIC1_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 3);    /* P13 is used as TMIOC1 input */ \
}while(0)

/* ToDo: You can allocate the TMIOD1 to P11, P13, or P10 with PIOR37 and PIOR36 register */
#define TMID1_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 1);    /* P11 is used as TMIOD1 input */ \
}while(0)

//#define TMOA0_PORT_SETTING() do{ \
//        PORT->PCLR1  =  (1 << 7);    /* P17 output low level */ \
//        PORT->PM1   &= ~(1 << 7);    /* P17 is used as TMIOA0 output */ \
//        PORT->POM1  &= ~(1 << 7);    /* P17 is push-pull output mode */ \
//}while(0)

#define TMOB0_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 4);    /* P14 output low level */ \
        PORT->PM1   &= ~(1 << 4);    /* P14 is used as TMIOB0/U+ output */ \
        PORT->POM1  &= ~(1 << 4);    /* P14 is push-pull output mode */ \
}while(0)

#define TMOC0_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 6);    /* P16 output low level */ \
        PORT->PM1   &= ~(1 << 6);    /* P16 is used as TMIOC0 output */ \
        PORT->POM1  &= ~(1 << 6);    /* P16 is push-pull output mode */ \
}while(0)

#define TMOD0_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 5);    /* P15 output low level */ \
        PORT->PM1   &= ~(1 << 5);    /* P15 is used as TMIOD0/U- output */ \
        PORT->POM1  &= ~(1 << 5);    /* P15 is push-pull output mode */ \
}while(0)

#define TMOA1_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 2);    /* P12 output low level */ \
        PORT->PM1   &= ~(1 << 2);    /* P12 is used as TMIOA1/V+ output */ \
        PORT->POM1  &= ~(1 << 2);    /* P12 is push-pull output mode */ \
}while(0)

#define TMOB1_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 0);    /* P10 output low level */ \
        PORT->PM1   &= ~(1 << 0);    /* P10 is used as TMIOB1/W+ output */ \
        PORT->POM1  &= ~(1 << 0);    /* P10 is push-pull output mode */ \
        PORT->PMC1  &= ~(1 << 0);    /* P10 is digital function */ \
}while(0)

#define TMOC1_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 3);    /* P13 output low level */ \
        PORT->PM1   &= ~(1 << 3);    /* P13 is used as TMIOC1/V- output */ \
        PORT->POM1  &= ~(1 << 3);    /* P13 is push-pull output mode */ \
}while(0)

#define TMOD1_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 1);    /* P11 output low level */ \
        PORT->PM1   &= ~(1 << 1);    /* P11 is used as TMIOD1/W- output */ \
        PORT->POM1  &= ~(1 << 1);    /* P11 is push-pull output mode */ \
        PORT->PMC1  &= ~(1 << 1);    /* P11 is digital function */ \
}while(0)

#define MTR_PORT_UP_SET() do{ \
        PORT->PSET1  =  (1 << 4);    /* TMIOB0/U+ output H level */ \
}while(0)

#define MTR_PORT_UP_CLR() do{ \
        PORT->PCLR1  =  (1 << 4);    /* TMIOB0/U+ output L level */ \
}while(0)

#define MTR_PORT_VP_SET() do{ \
        PORT->PSET1  =  (1 << 2);    /* TMIOA1/V+ output H level */ \
}while(0)

#define MTR_PORT_VP_CLR() do{ \
        PORT->PCLR1  =  (1 << 2);    /* TMIOA1/V+ output L level */ \
}while(0)

#define MTR_PORT_WP_SET() do{ \
        PORT->PSET1  =  (1 << 0);    /* TMIOB1/W+ output H level */ \
}while(0)

#define MTR_PORT_WP_CLR() do{ \
        PORT->PCLR1  =  (1 << 0);    /* TMIOB1/W+ output L level */ \
}while(0)

#define MTR_PORT_UN_SET() do{ \
        PORT->PSET1  =  (1 << 5);    /* TMIOD0/U- output H level */ \
}while(0)

#define MTR_PORT_UN_CLR() do{ \
        PORT->PCLR1  =  (1 << 5);    /* TMIOD0/U- output L level */ \
}while(0)

#define MTR_PORT_VN_SET() do{ \
        PORT->PSET1  =  (1 << 3);    /* TMIOC1/V- output H level */ \
}while(0)

#define MTR_PORT_VN_CLR() do{ \
        PORT->PCLR1  =  (1 << 3);    /* TMIOC1/V- output L level */ \
}while(0)

#define MTR_PORT_WN_SET() do{ \
        PORT->PSET1  =  (1 << 1);    /* TMIOD1/W- output H level */ \
}while(0)

#define MTR_PORT_WN_CLR() do{ \
        PORT->PCLR1  =  (1 << 1);    /* TMIOD1/W- output L level */ \
}while(0)

/* ================================================================================================================== */
/* ================                                      CLKBUZ                                      ================ */
/* ================================================================================================================== */

/**
  * @brief CLKBUZ Port Setting
  */
#define CLKBUZ0_PORT_SETTING() do{ \
        PORT->PCLR7  =  (1 << 2);   /* P72 output low level */ \
        PORT->PM7   &= ~(1 << 2);   /* P72 is used as CLKBUZ0 output */ \
}while(0)

#define CLKBUZ1_PORT_SETTING() do{ \
        PORT->PCLR1  =  (1 << 5);   /* P15 output low level */ \
        PORT->PM1   &= ~(1 << 5);   /* P15 is used as CLKBUZ1 output */ \
}while(0)

/* ================================================================================================================== */
/* ================                                        ADC                                       ================ */
/* ================================================================================================================== */

/**
  * @brief ADC Port Setting (Alternate to fixed ports)
  */

/* ToDo: Please comment out the following unused ANIx setting according to your application needs. */
#define ADC_PORT_SETTING() do{ \
        PORT->PMC2 |= (1 << 0);   /* Set ANI0(P20) pin: It is necessary for ADC_VREF_AVREFP_AVREFM, used as AVREFP */ \
        PORT->PMC2 |= (1 << 1);   /* Set ANI1(P21) pin: It is necessary for ADC_VREF_AVREFP_AVREFM, used as AVREFM */ \
        PORT->PMC2 |= (1 << 2);   /* Set ANI2(P22) pin */ \
        PORT->PMC2 |= (1 << 3);   /* Set ANI3(P23) pin */ \
        PORT->PMC1 |= (1 << 1);   /* Set ANI8(P11) pin */ \
        PORT->PMC1 |= (1 << 0);   /* Set ANI9(P10) pin */ \
        PORT->PMC0 |= (1 << 1);   /* Set ANI10(P01) pin */ \
        PORT->PMC0 |= (1 << 0);   /* Set ANI11(P00) pin */ \
}while(0)

/* ================================================================================================================== */
/* ================                                        DAC                                       ================ */
/* ================================================================================================================== */

/**
  * @brief DAC Port Setting (Alternate to fixed ports)
  */

#define DAC0_PORT_SETTING() do{ \
        PORT->PMC2 |= (1 << 2);   /* Set ANO0(P22) pin */ \
}while(0)

#define DAC1_PORT_SETTING() do{ \
        PORT->PMC2 |= (1 << 3);   /* Set ANO1(P23) pin */ \
}while(0)

/* ================================================================================================================== */
/* ================                                        CMP                                       ================ */
/* ================================================================================================================== */

/**
  * @brief CMP Port Setting (Analog input alternate to fixed ports, digital output alternate to 2 group ports)
  */

#if 1
/* ToDo: You can allocate the VCOUT0 to P12 or P73 with PIOR20 register */
#define VCOUT0_PORT_SETTING() do { \
        PORT->PIOR3 |=  (1 << 1);  /* VCOUT0 output enable */ \
        PORT->PIOR2 &= ~(1 << 0);  /* allocate VCOUT0 to P12 */ \
        PORT->PCLR1 =  (1 << 2);  /* P12 output low level */ \
        PORT->PM1  &= ~(1 << 2);  /* VCOUT0 output to P12 */ \
        PORT->PMC1 &= ~(1 << 2);  /* P12 is digital function */ \
}while(0)
#else
/* ToDo: You can allocate the VCOUT0 to P120 or P73 with PIOR20 register */
#define VCOUT0_PORT_SETTING() do { \
        PORT->PIOR3 |=  (1 << 1);  /* VCOUT0 output enable */ \
        PORT->PIOR2 |=  (1 << 0);  /* allocate VCOUT0 to P73 */ \
        PORT->PCLR7  =  (1 << 3);  /* P73 output low level */ \
        PORT->PM7   &= ~(1 << 3);  /* VCOUT0 output to P73 */ \
}while(0)
#endif

#if 1
/* ToDo: You can allocate the VCOUT1 to P31 or P70 with PIOR21 register */
#define VCOUT1_PORT_SETTING() do { \
        PORT->PIOR3 |=  (1 << 2);  /* VCOUT1 output enable */ \
        PORT->PIOR2 |= (1 << 1);  /* allocate VCOUT1 to P72 */ \
        PORT->PCLR7  =  (1 << 2);  /* P72 output low level */ \
        PORT->PM7   &= ~(1 << 2);  /* VCOUT1 output to P72 */ \
}while(0)
#else
#define VCOUT1_PORT_SETTING() do { \
        PORT->PIOR3 |=  (1 << 2);  /* VCOUT1 output enable */ \
        PORT->PIOR2 |=  (1 << 1);  /* allocate VCOUT1 to P70 */ \
        PORT->PCLR7  =  (1 << 0);  /* P70 output low level */ \
        PORT->PM7   &= ~(1 << 0);  /* VCOUT1 output to P70 */ \
}while(0)
#endif

/* ToDo: Please comment out the VREF0 setting if don't used VREF0 as negative(-) side input of CMP */
#define CMP0_PORT_SETTING() do{ \
        PORT->PMC2  |= (1 << 2);  /* Set VCIN0(P22) pin */ \
        VCOUT0_PORT_SETTING();    /* ToDo: Please delete me if you don't output VCOUT0 signal to port */ \
}while(0)

/* ToDo: Please comment out the VCINxx setting if don't used it as positive(+) side input of CMP */
#define CMP1_PORT_SETTING() do{ \
        PORT->PMC0  |= (1 << 0);  /* Set VCIN10(P00) pin */ \
        PORT->PMC0  |= (1 << 1);  /* Set VCIN11(P01) pin */ \
        PORT->PMC2  |= (1 << 0);  /* Set VCIN12(P20) pin */ \
        PORT->PMC2  |= (1 << 1);  /* Set VCIN13(P21) pin */ \
        VCOUT1_PORT_SETTING();    /* ToDo: Please delete me if you don't output VCOUT1 signal to port */ \
}while(0)

/* ================================================================================================================== */
/* ================                                        PGA                                       ================ */
/* ================================================================================================================== */

/**
  * @brief PGA Port Setting (Alternate to fixed ports)
  */

#define PGA0O_PORT_SETTING() do { \
        PORT->PMC2 |= (1 << 0);   /* PGA0O output to P20 */ \
}while(0)

#define PGA0IN_PORT_SETTING() do { \
        PORT->PMC2 |= (1 << 2);  /* PGA0IN input from P22 */ \
}while(0)

#define PGA0GND_PORT_SETTING() do { \
        PORT->PMC2 |= (1 << 3);  /* PGA0GND input from P23 */ \
}while(0)

#define PGA1IN_PORT_SETTING() do { \
        PORT->PMC0 |= (1 << 1);  /* PGA1IN input from P01 */ \
}while(0)


/* ================================================================================================================== */
/* ================                                       SCI0                                       ================ */
/* ================================================================================================================== */
/* ToDo: You can allocate the TXD0, RXD0, SCLK00, SDI00, SDO00, SCL00 and SDA00 to the following ports with PIOR register */
/* PIOR35  PIOR34  PIOR01 :  TXD0  RXD0  SCLK00  SDO00  SDI00  SCL00  SDA00
 *    0       0       0   :  P51   P50   P30     P51    P50    P30    P50   # default setting
 *    0       0       1   :  P17   P16   P55     P17    P16     -      -
 *    0       1       X   :  P40   P137   -       -      -      -      -
 *    1       X       X   :  P12   P11    -       -      -      -      -
*/
/**
  * @brief UART0 Port Setting (Alternative to 4 group ports) 
  */

#if 1
/* ToDo: You can allocate the TXD0 to P51, P17, P40 or P12 with PIOR35, PIOR43 and PIOR01 register */
#define TXD0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate TXD0 to P51 */ \
        PORT->PIOR3 &= ~(1 << 4);    /* allocate TXD0 to P51 */ \
        PORT->PIOR3 &= ~(1 << 4);    /* allocate TXD0 to P51 */ \
        PORT->PSET5  =  (1 << 1);    /* P51 output high level */ \
        PORT->PM5   &= ~(1 << 1);    /* P51 is used as TXD0 output */ \
        PORT->POM5  &= ~(1 << 1);    /* P51 is push-pull output mode */ \
}while(0)

/* ToDo: You can allocate the RXD0 to P50, P16, P137 or P11 with PIOR35, PIOR43 and PIOR01 register */
#define RXD0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate RXD0 to P50 */ \
        PORT->PIOR3 &= ~(1 << 4);    /* allocate TXD0 to P51 */ \
        PORT->PIOR3 &= ~(1 << 4);    /* allocate TXD0 to P51 */ \
        PORT->PM5   |=  (1 << 0);    /* P50 is used as RXD0 input */ \
}while(0)

#else
/* ToDo: You can allocate the TXD0 to P51, P17, P40 or P12 with PIOR35, PIOR43 and PIOR01 register */
#define TXD0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 1);    /* allocate TXD0 to P17 */ \
        PORT->PSET1  =  (1 << 7);    /* P17 output high level */ \
        PORT->PM1   &= ~(1 << 7);    /* P17 is used as TXD0 output */ \
        PORT->POM1  &= ~(1 << 7);    /* P17 is push-pull output mode */ \
}while(0)

/* ToDo: You can allocate the RXD0 to P50, P16, P137 or P11 with PIOR35, PIOR43 and PIOR01 register */
#define RXD0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 1);    /* allocate RXD0 to P16 */ \
        PORT->PM1   |=  (1 << 6);    /* P16 is used as RXD0 input */ \
}while(0)
#endif

/**
  * @brief SPI00 Port Setting (Alternative to 2 group ports)
  */

#define SS00_PORT_SETTING() do{ \
        PORT->PM7  |=  (1 << 3);     /* P73 is used as SS00 input */ \
}while(0)

#define SS00_PORT_SET() do{ \
        PORT->PSET7 =  (1 << 3);     /* P73 output high level */ \
}while(0)

#define SS00_PORT_CLR() do{ \
        PORT->PCLR7 =  (1 << 3);     /* P73 output low level */ \
}while(0)

/* ToDo: You can allocate the SCLK00 to P30 or P55 with PIOR01 register */
#define SCLKI00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);     /* allocate SCLKI00 to P30 */ \
        PORT->PM7   |=  (1 << 2);     /* P72 is used as SCLK00 input */ \
}while(0)

/* ToDo: You can allocate the SCLK00 to P30 or P55 with PIOR01 register */
#define SCLKO00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate SCLKI00 to P72 */ \
        PORT->PSET7  =  (1 << 2);    /* P72 output high level */ \
        PORT->PM7   &= ~(1 << 2);    /* P72 is used as SCLK00 output */ \
        PORT->POM7  &= ~(1 << 2);    /* P72 is push-pull output mode */ \
}while(0)

/* ToDo: You can allocate the SDO00 to P51 or P17 with PIOR01 register */
#define SDO00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate SDO00 to P51 */ \
        PORT->PSET5  =  (1 << 1);    /* P51 output high level */ \
        PORT->PM5   &= ~(1 << 1);    /* P51 is used as SDO00 output */ \
        PORT->POM5  &= ~(1 << 1);    /* P51 is push-pull output mode */ \
}while(0)

/* ToDo: You can allocate the SDI00 to P50 or P16 with PIOR01 register */
#define SDI00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate SDI00 to P50 */ \
        PORT->PM5   |=  (1 << 0);    /* P50 is used as SDI00 input */ \
}while(0)

/**
  * @brief IIC00 Port Setting (Alternative to fixed port)
  */

#define SCL00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate SCL00 to P72 */ \
        PORT->PSET7  =  (1 << 2);    /* P30 output high level */ \
        PORT->PM7   &= ~(1 << 2);    /* P30 is used as SCL00 output */ \
        PORT->POM7  |=  (1 << 2);    /* P30 is N-ch open-drain output mode */ \
}while(0)

#define SDA00_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 1);    /* allocate SDA00 to P50 */ \
        PORT->PSET5  =  (1 << 0);    /* P50 output high level */ \
        PORT->PM5   &= ~(1 << 0);    /* P50 is used as SDA00 inout */ \
        PORT->POM5  |=  (1 << 0);    /* P50 is N-ch open-drain output mode */ \
}while(0)


/* ================================================================================================================== */
/* ================                                       SCI2                                       ================ */
/* ================================================================================================================== */

/**
  * @brief UART2 Port Setting (Alternative to 2 group ports for 64pin product) 
  */

/* ToDo: You can allocate the TXD2 to P13 or P77 with PIOR01 register */
#define TXD2_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate TXD2 to P13 */ \
        PORT->PSET1  =  (1 << 3);    /* P13 output high level */ \
        PORT->PM1   &= ~(1 << 3);    /* P13 is used as TXD2 output */ \
        PORT->POM1  &= ~(1 << 3);    /* P13 is push-pull output mode */ \
        PORT->PMC1  &= ~(1 << 3);    /* P13 is digital function */ \
}while(0)

/* ToDo: You can allocate the RXD2 to P14 or P76 with PIOR01 register */
#define RXD2_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate RXD2 to P14 */ \
        PORT->PM1   |=  (1 << 4);    /* P14 is used as RXD2 input */ \
        PORT->PMC1  &= ~(1 << 4);    /* P14 is digital function */ \
}while(0)

/**
  * @brief SPI20 Port Setting (Alternative to fixed port)
  */

/* ToDo: You can allocate the SS20 to any desired pins internal selected*/
#define SS20_PORT_SETTING() do{ \
}while(0)
//        PORT->PSET1 =  (1 << 6);     /* P16 output high level */ \
//        PORT->PM1  &= ~(1 << 6);     /* P16 is used as SS20 output */ \

/* ToDo: You can allocate the SS20 to any desired pins */
#define SS20_PORT_SET() do{ \
}while(0)
//        PORT->PSET1 =  (1 << 6);     /* P16 output high level */ \

/* ToDo: You can allocate the SS20 to any desired pins */
#define SS20_PORT_CLR() do{ \
}while(0)
//        PORT->PCLR1 =  (1 << 6);      /* P16 output low level */ \

#define SCLKI20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SCLK20 to P15 */ \
        PORT->PM1   |=  (1 << 5);    /* P15 is used as SCLK20 input */ \
}while(0)

#define SCLKO20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SCLK20 to P15 */ \
        PORT->PSET1  =  (1 << 5);    /* P15 output high level */ \
        PORT->PM1   &= ~(1 << 5);    /* P15 is used as SCLK20 output */ \
        PORT->POM1  &= ~(1 << 5);    /* P15 is push-pull output mode */ \
}while(0)

#define SDO20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SDO20 to P15 */ \
        PORT->PSET1  =  (1 << 3);    /* P13 output high level */ \
        PORT->PM1   &= ~(1 << 3);    /* P13 is used as SDO20 output */ \
        PORT->POM1  &= ~(1 << 3);    /* P13 is push-pull output mode */ \
}while(0)

#define SDI20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SDI20 to P14 */ \
        PORT->PM1   |=  (1 << 4);    /* P14 is used as SDI20 input */ \
}while(0)

/**
  * @brief IIC20 Port Setting (Alternative to fixed port)
  */

#define SCL20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SCL20 to P15 */ \
        PORT->PSET1  =  (1 << 5);    /* P15 output high level */ \
        PORT->PM1   &= ~(1 << 5);    /* P15 is used as SCL20 output */ \
        PORT->POM1  |=  (1 << 5);    /* P15 is N-ch open-drain output mode */ \
}while(0)

#define SDA20_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 6);    /* allocate SDA20 to P14 */ \
        PORT->PSET1  =  (1 << 4);    /* P14 output high level */ \
        PORT->PM1   &= ~(1 << 4);    /* P14 is used as SDA20 inout */ \
        PORT->POM1  |=  (1 << 4);    /* P14 is N-ch open-drain output mode */ \
}while(0)


/* ================================================================================================================== */
/* ================                                       SPI                                        ================ */
/* ================================================================================================================== */

/**
  * @brief SPI Port Setting (Alternative to fixed port)
  */

/* ToDo: Please define one of the following macro and comment out others accroding to your application */
#define SPIHS0_SLAVE_PORT
//#define SPIHS0_MASTER_PORT

#ifdef SPIHS0_SLAVE_PORT
#define NSS0_PORT_SETTING() do{ \
				SPIHS0->SPIM &= ~_0020_SPI_NSS_ENABLE;   /* NSS use internal GND */ \
}while(0)

#define NSS0_PORT_SET() do{ \
	      SPIHS0->SPIM &= ~_0020_SPI_NSS_ENABLE;   /* NSS use internal GND */ \
        __NOP(); \
}while(0)

#define NSS0_PORT_CLR() do{ \
        __NOP(); \
}while(0)

#define SCK0_PORT_SETTING() do{ \
        PORT->PMC1 &= ~(1 << 0);      /* P10 is digital function */ \
        PORT->PM1  |=  (1 << 0);      /* P10 is used as SCK input */ \
}while(0)

#define MOSI0_PORT_SETTING() do{ \
        PORT->PM1  |=  (1 << 2);      /* P12 is used as MOSI input */ \
}while(0)

#define MISO0_PORT_SETTING() do{ \
        PORT->PMC1 &= ~(1 << 1);      /* P11 is digital function */ \
        PORT->PCLR1 =  (1 << 1);      /* P11 output low level */ \
        PORT->PM1  &= ~(1 << 1);      /* P11 is used as MISO output */ \
        PORT->POM1 &= ~(1 << 1);      /* P11 is normal output mode */ \
}while(0)
#endif

#ifdef SPIHS0_MASTER_PORT
#define NSS0_PORT_SETTING() do{ \
        SPIHS0->SPIM &= ~_0020_SPI_NSS_ENABLE;   /* NSS use internal GND */ \
}while(0)

#define NSS0_PORT_SET() do{ \
        SPIHS0->SPIM &= ~_0020_SPI_NSS_ENABLE;   /* NSS use internal GND */ \
}while(0)

#define NSS0_PORT_CLR() do{ \
//        PORT->PCLR14 =  (1 << 7);     /* P147 output low level */ \
}while(0)

#define SCK0_PORT_SETTING() do{ \
        PORT->PMC1 &= ~(1 << 0);      /* P10 is digital function */ \
        PORT->PCLR1 =  (1 << 0);      /* P10 output low level */ \
        PORT->PM1  &= ~(1 << 0);      /* P10 is used as SCK output */ \
        PORT->POM1 &= ~(1 << 0);      /* P10 is normal output mode */ \
}while(0)

#define MOSI0_PORT_SETTING() do{ \
        PORT->PCLR1 =  (1 << 2);      /* P12 output low level */ \
        PORT->PM1  &= ~(1 << 2);      /* P12 is used as MOSI output */ \
        PORT->POM1 &= ~(1 << 2);      /* P12 is normal output mode */ \
}while(0)

#define MISO0_PORT_SETTING() do{ \
        PORT->PMC1 &= ~(1 << 1);      /* P11 is digital function */ \
        PORT->PM1  |=  (1 << 1);      /* P11 is used as MISO input */ \
}while(0)
#endif


/* ================================================================================================================== */
/* ================                                       LIN                                       ================ */
/* ================================================================================================================== */

/**
  * @brief LIN Port Setting (Alternative to 2 group ports)
  */

#if 1
/* ToDo: You can allocate the LTXD0 to P00 or P72 with PIOR05 register */
#define LTXD0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 5);    /* allocate LTXD0 to P00 */ \
        PORT->PSET0  =  (1 << 0);    /* P00 output high level */ \
        PORT->PM0   &= ~(1 << 0);    /* P00 is used as LTXD0 output */ \
        PORT->PMC0  &= ~(1 << 0);    /* P00 is used as digital */ \
}while(0)

/* ToDo: You can allocate the LRXD0 to P01 or P73 with PIOR05 register */
#define LRXD0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 5);    /* allocate LRXD0 to P01 */ \
        PORT->PCLR0  =  (1 << 1);    /* P01 output low level */ \
        PORT->PM0   &= ~(1 << 1);    /* P01 is used as LRXD0 inout */ \
        PORT->PMC0  &= ~(1 << 1);    /* P01 is used as digital */ \
}while(0)

#else
/* ToDo: You can allocate the LTXD0 to P00 or P72 with PIOR05 register */
#define LTXD0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 5);    /* allocate LTXD0 to P72 */ \
        PORT->PSET7  =  (1 << 2);    /* P72 output high level */ \
        PORT->PM7   &= ~(1 << 2);    /* P72 is used as LTXD0 output */ \
}while(0)

/* ToDo: You can allocate the LRXD0 to P01 or P73 with PIOR05 register */
#define LRXD0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 5);    /* allocate LRXD0 to P73 */ \
        PORT->PCLR7  =  (1 << 3);    /* P73 output low level */ \
        PORT->PM7   &= ~(1 << 3);    /* P73 is used as LRXD0 inout */ \
}while(0)
#endif


/* ================================================================================================================== */
/* ================                                       IICA                                       ================ */
/* ================================================================================================================== */

/**
  * @brief IICA0 Port Setting (Alternative to 2 group ports)
  */

#if 0
/* ToDo: You can allocate the SCLA0 to P31 or P14 with PIOR02 register */
#define SCLA0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 2);    /* allocate SCLA0 to P31 */ \
        PORT->PCLR3  =  (1 << 1);    /* P31 output low level */ \
        PORT->PM3   &= ~(1 << 1);    /* P31 is used as SCLA0 output */ \
        PORT->POM3  |=  (1 << 1);    /* P31 is N-ch open-drain output mode */ \
}while(0)

/* ToDo: You can allocate the SDAA0 to P74 or P15 with PIOR02 register */
#define SDAA0_PORT_SETTING() do{ \
        PORT->PIOR0 &= ~(1 << 2);    /* allocate SDAA0 to P74 */ \
        PORT->PCLR7  =  (1 << 4);    /* P74 output low level */ \
        PORT->PM7   &= ~(1 << 4);    /* P74 is used as SDAA0 inout */ \
        PORT->POM7  |=  (1 << 4);    /* P74 is N-ch open-drain output mode */ \
}while(0)

#else
/* ToDo: You can allocate the SCLA0 to P60 or P14 with PIOR02 register */
#define SCLA0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 2);    /* allocate SCLA0 to P14 */ \
        PORT->PCLR1  =  (1 << 4);    /* P14 output low level */ \
        PORT->PM1   &= ~(1 << 4);    /* P14 is used as SCLA0 output */ \
        PORT->POM1  |=  (1 << 4);    /* P14 is N-ch open-drain output mode */ \
}while(0)

/* ToDo: You can allocate the SDAA0 to P61 or P15 with PIOR02 register */
#define SDAA0_PORT_SETTING() do{ \
        PORT->PIOR0 |=  (1 << 2);    /* allocate SDAA0 to P15 */ \
        PORT->PCLR1  =  (1 << 5);    /* P15 output low level */ \
        PORT->PM1   &= ~(1 << 5);    /* P15 is used as SDAA0 inout */ \
        PORT->POM1  |=  (1 << 5);    /* P15 is N-ch open-drain output mode */ \
}while(0)
#endif


/* ================================================================================================================== */
/* ================                                       INTP                                       ================ */
/* ================================================================================================================== */
/**
  * @brief INTP Port Setting 
  */
//#define INTP0_PORT_SETTING() do{ \
//        PORT->PM13  |=  (1 << 6);   /* P136 is used as INTP0 input */ \
//}while(0)

#define INTP1_PORT_SETTING() do{ \
        PORT->PM5   |=  (1 << 0);    /* P50 is used as INTP1 input */ \
}while(0)

#define INTP2_PORT_SETTING() do{ \
        PORT->PM5   |=  (1 << 1);    /* P51 is used as INTP2 input */ \
}while(0)

#define INTP3_PORT_SETTING() do{ \
        PORT->PM7   |=  (1 << 3);    /* P73 is used as INTP3 input */ \
}while(0)

#define INTP4_PORT_SETTING() do{ \
        PORT->PM7   |=  (1 << 2);    /* P72 is used as INTP4 input */ \
}while(0)

#define INTP5_PORT_SETTING() do{ \
        PORT->PM1   |=  (1 << 2);    /* P12 is used as INTP5 input */ \
}while(0)

#define INTP6_PORT_SETTING() do{ \
        PORT->PM7  |=  (1 << 0);     /* P70 is used as INTP6 input */ \
}while(0)

#define INTP7_PORT_SETTING() do{ \
        PORT->PM7  |=  (1 << 2);     /* P72 is used as INTP7 input */ \
}while(0)

#define INTP8_PORT_SETTING() do{ \
        PORT->PM0   |=  (1 << 0);    /* P00 is used as INTP8 input */ \
}while(0)

// #define INTP9_PORT_SETTING() do{ \
//         PORT->PM7   |=  (1 << 5);    /* P75 is used as INTP9 input */ \
// }while(0)

#define INTP10_PORT_SETTING() do{ \
        PORT->PM7   |=  (1 << 1);    /* P01 is used as INTP10 input */ \
}while(0)

#define INTP11_PORT_SETTING() do{ \
        PORT->PM2   |=  (1 << 0);    /* P20 is used as INTP11 input */ \
}while(0)

/** @} */ /* End of group Peripherals_Port_Setting_Definations */

#endif
