/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    dma.h
* @brief   This file implements device driver for DTC module.
* @version 1.0.0  
* @date    2022/2/25
***********************************************************************************************************************/
#ifndef DMA_H
#define DMA_H

/***********************************************************************************************************************
Macro definitions (Register bit)
***********************************************************************************************************************/
/*
    Bit Position of DMACR
*/
#define SZ_Pos       6
#define RPTINT_Pos   5
#define CHNE_Pos     4
#define DAMOD_Pos    3
#define SAMOD_Pos    2
#define RPTSEL_Pos   1
#define MODE_Pos     0

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/
typedef enum {
   DMA_VECTOR_INTP0 	=  1U,
   DMA_VECTOR_INTP1 	=  2U,
   DMA_VECTOR_INTP2 	=  3U,
   DMA_VECTOR_INTP3 	=  4U,
   DMA_VECTOR_INTP4 	=  5U,
   DMA_VECTOR_INTP5 	=  6U,
   DMA_VECTOR_INTP6 	=  7U,
   DMA_VECTOR_INTP7 	=  8U,
   DMA_VECTOR_ADC   	= 10U,
   DMA_VECTOR_SR0       = 11U,
   DMA_VECTOR_SPI01     = 11U,
   DMA_VECTOR_IIC01     = 11U,
   DMA_VECTOR_ST0       = 12U,
   DMA_VECTOR_SPI00     = 12U,
   DMA_VECTOR_IIC00     = 12U,
   DMA_VECTOR_LINR      = 13U,
   DMA_VECTOR_LINT      = 14U,
   DMA_VECTOR_SR2       = 15U,
   DMA_VECTOR_SPI21     = 15U,
   DMA_VECTOR_IIC21     = 15U,
   DMA_VECTOR_ST2       = 16U,
   DMA_VECTOR_SPI20     = 16U,
   DMA_VECTOR_IIC20     = 16U,
   DMA_VECTOR_IICA      = 17U,
   DMA_VECTOR_SPIHS0    = 18U,
   DMA_VECTOR_TM4_CH0   = 19U,
   DMA_VECTOR_TM4_CH1   = 20U,
   DMA_VECTOR_TM4_CH2   = 21U,
   DMA_VECTOR_TM4_CH3   = 22U,
   DMA_VECTOR_FLASH     = 24U,
   DMA_VECTOR_TMM_UNF   = 25U,
   DMA_VECTOR_TMC_OVF   = 26U,
   DMA_VECTOR_TMM_IMFA0 = 27U,
   DMA_VECTOR_TMM_IMFB0 = 28U,
   DMA_VECTOR_TMM_IMFC0 = 29U,
   DMA_VECTOR_TMM_IMFD0 = 30U,
   DMA_VECTOR_TMM_IMFA1 = 31U,
   DMA_VECTOR_TMM_IMFB1 = 32U,
   DMA_VECTOR_TMM_IMFC1 = 33U,
   DMA_VECTOR_TMM_IMFD1 = 34U,
   DMA_VECTOR_TMB_IMFA  = 35U,
   DMA_VECTOR_TMB_IMFB  = 36U,
   DMA_VECTOR_TMA_UNF   = 37U,
   DMA_VECTOR_CMP0      = 38U,
   DMA_VECTOR_CMP1      = 39U 

} dma_vector_t;

typedef enum {
	DMA_SIZE_BYTE, 				/*  8bits transfer */ 
	DMA_SIZE_HALF,  			/* 16bits transfer */
	DMA_SIZE_WORD   			/* 32bits transfer */
} dma_size_t;

typedef enum {
	DMA_MODE_NORMAL, 			/* Normal mode */ 
	DMA_MODE_REPEAT   			/* Repeat mode */
} dma_mode_t;

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
#if 1
void DMA_Start(dma_vector_t vect_num, uint8_t ctrl_data_num, dma_mode_t mode, dma_size_t size, uint16_t count, void *src_adr, void *dst_adr);
#else
void DMA_Start(dma_vector_t vect_num, uint8_t ctrl_data_num, DMAVEC_CTRL_Type ctrl_data);
#endif
void DMA_Enable(dma_vector_t vect_num);
void DMA_Stop(dma_vector_t vect_num);
void DMA_Trigger(dma_vector_t vect_num);
/* Start user code for function. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#endif
