/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    pga.h
* @brief   This file implements device driver for PGA module.
* @version 1.0.0
* @date    2022/2/25
***********************************************************************************************************************/
#ifndef PGA_H
#define PGA_H

/***********************************************************************************************************************
Macro definitions (Register bit)
***********************************************************************************************************************/
/*
    PGA control register (PGACTL) 
*/
/* PGA operation selection (PGAEN) */
#define _00_PGA_OPERATION_STOP                        (0x00U) /* PGA operation stopped */
#define _80_PGA_OPERATION_ENABLE                      (0x80U) /* PGA operation enabled */
/* GND selection of feedback resistance of the programmable gain amplifier (PVRVS) */
#define _00_PGA_VSS_SELECTED                          (0x00U) /* selects VSS */
#define _08_PGA_PGAGND_SELECTED                       (0x08U) /* selects PGAGND */
/* Selected PGA amplification (PGAVG1,PGAVG0) */
#define _00_PGA_AMPLIFICATION_X4                      (0x00U) /* *4 is selected as the gain */
#define _01_PGA_AMPLIFICATION_X8                      (0x01U) /* *8 is selected as the gain */
#define _02_PGA_AMPLIFICATION_X10                     (0x02U) /* *10 is selected as the gain */
#define _03_PGA_AMPLIFICATION_X12                     (0x03U) /* *12 is selected as the gain */
#define _04_PGA_AMPLIFICATION_X14                     (0x04U) /* *14 is selected as the gain */
#define _05_PGA_AMPLIFICATION_X16                     (0x05U) /* *16 is selected as the gain */
#define _06_PGA_AMPLIFICATION_X32                     (0x06U) /* *32 is selected as the gain */
#define _07_PGA_AMPLIFICATION_X32                     (0x07U) /* *32 is selected as the gain */

#define PGAEN                                         (0x80U) /* PGA operation enabled */
#define PGA0EN                                        (0x40U) /* PGAOUT/P20 output enabled */

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/
typedef enum 
{
    PGA_CHANNEL_0 = 0x00U,      // PGA0IN & PGA0GND
    PGA_CHANNEL_1 = 0x20U,      // PGA1IN & PGA1GND
} pga_channel_t;                // PGAINSEL

typedef enum 
{
    PGA_VSS_INPUT       = 0x00U,      // Select VSS as GROUND of feedback resistance string 
    PGA_PGAxGND_INPUT    = 0x08U       //only for PGA0, Select PGA0GND as GROUND of feedback resistance string,
} pga_mode_t;                   // PGAMODE

typedef enum 
{
    HALF_VDD     = 0U,          // 1/2 Vdd as vref
    INNER_1P45V  = 1U,          // Inner 1.45V
    INNER_2P4V   = 2U           // Inner 2.4V
} pga_vref_t;                   // PGAVREF & VREFSEL

typedef enum 
{
    PGA_GAIN_X1   = 0U,     // X1
    PGA_GAIN_X2P5 = 1U,     // X2.5
    PGA_GAIN_X4   = 2U,     // X4
    PGA_GAIN_X8   = 3U,     // X8
    PGA_GAIN_X10  = 4U,     // X10
    PGA_GAIN_X16  = 5U,     // X16
    PGA_GAIN_X32  = 6U      // X32
} pga_gain_t;               // PGA0CTL->PGAVG[2:0]

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
void PGA_Init(pga_channel_t ch, pga_mode_t mode, pga_vref_t vref, pga_gain_t gain);
void PGAOUT_Enable(void);
void PGAOUT_Disable(void);
void PGA_Start(void);
void PGA_Stop(void);
/* Start user code for function. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#endif

