/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    lin_user.c
* @brief   This file implements device driver for LIN module.
* @version 1.0.0 
* @date    2022/2/25
***********************************************************************************************************************/

#include "userdefine.h"
#include "BAT32G113.h"
#include "lin.h"

void IRQ13_Handler(void) __attribute__((alias("LIN_Transmission_interrupt")));
void IRQ14_Handler(void) __attribute__((alias("LIN_Reception_interrupt")));
void IRQ15_Handler(void) __attribute__((alias("LIN_Status_interrupt")));

#ifdef LIN_MASTER
/***********************************************************************************************************************
* Function Name: LIN_Transmission_interrupt
* Description  : This function is LIN Transmission interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Transmission_interrupt(void)
{

  uint8_t transmit_header_flag;
  uint8_t transmit_response_flag;

  INTC_ClearPendingIRQ(LINT_IRQn);
  transmit_header_flag=LIN->LST & 0x80;
  transmit_response_flag=LIN->LST & 0x01;
  
    if(transmit_header_flag)
  {    
    /*LIN->LIE&=0xF7; */   /*Disable header transmit interrupt*//*revision 1.01*/
    LIN->LST&=0X7F;    /*clear successful header reception flag */  
    /*LIN->LIE|=0x80; */   /*enable header transmit interrupt*//*revision 1.01*/
    } 
   
  if(transmit_response_flag)
  { 
    LIN->LST&=0xFE;      
  }


}


/***********************************************************************************************************************
* Function Name: LIN_Reception_interrupt
* Description  : This function is RLIN0 Reception interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Reception_interrupt(void)
{  

  INTC_ClearPendingIRQ(LINR_IRQn);
  LIN->LST&=0xFD;    /*clear response reception successful flag*/
  LIN_Master_GetData();  /*get the reception data*/

}

#endif


#ifdef LIN_SLAVE
uint8_t GetIDbuffer;
uint8_t Slave_RxData1[8];            /*reception data store array*/
uint8_t Slave_RxData2[8];            /*reception data store array*/
uint8_t Slave_RxData3[8];            /*reception data store array*/
uint8_t Slave_TxData[]={0x55,0xC0};  /*Transmission data store array*/


/***********************************************************************************************************************
* Function Name: LIN_Transmission_interrupt
* Description  : This function is LIN Transmission interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Transmission_interrupt(void)
{
    
  INTC_ClearPendingIRQ(LINT_IRQn);
  LIN->LST&=0xFE; 

}


/***********************************************************************************************************************
* Function Name: LIN_Reception_interrupt
* Description  : This function is LIN Reception interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Reception_interrupt(void)
{  
    uint8_t receive_header_flag;
    uint8_t receive_reponse_flag;

    INTC_ClearPendingIRQ(LINR_IRQn);
    receive_header_flag=LIN->LST & 0x80;          /* get header reception flag */
    receive_reponse_flag=LIN->LST & 0X02;         /* get response rception flag*/

    GetIDbuffer=LIN->LIDB;

    if(receive_header_flag)  /* Header successful receive*/
    {
        /* LIN->LIE&=0xF7; */    /*Disable header interrupt*/ /*revision 1.01*/
        LIN->LST&=0X7F;    /*clear successful header reception flag*/

        switch(GetIDbuffer)
        {
            case 0x08: LIN_Slave_Receive(3);
                   break;
            case 0x49: LIN_Slave_Receive(3);
                   break;
            case 0xCA: LIN_Slave_Receive(3);
                   break;
            case 0x8B: LIN_Slave_Transmit(Slave_TxData,2);
                   break;  
            default:   LIN_Slave_NoResponse();
                   break;
        }
        /*LIN->LIE |= 0x80; */  /*enable header reception interrupt*//*revision 1.01*/
    }

    if(receive_reponse_flag)
    {
        /*LIN->LIE &= 0xFD; */      /* disable response reception interrupt*//*revision 1.01*/
        LIN->LST &= 0xFD;      /* clear response reception successful flag*/

        switch(GetIDbuffer)
        {
            case 0x08:  Get_response_RxData(Slave_RxData1);            
                      break;
            case 0x49:  Get_response_RxData(Slave_RxData2);              
                      break;
            case 0xCA:  Get_response_RxData(Slave_RxData3);             
                      break;
            default: break;
        }

        /*LIN->LIE|=0x02; */  /*enable reception interrupt*//*revision 1.01*/
    }
    LIN->LTRC=0x01;    /*enabled header reception interrupt*/
}


#endif

/***********************************************************************************************************************
* Function Name: LIN_Status_interrupt
* Description  : This function is LIN Status interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

void LIN_Status_interrupt(void)
{
    INTC_ClearPendingIRQ(LINS_IRQn);
    while(1U)
    {
        ;
    }
}


/***********************************************************************************************************************
* Function Name: LIN_Wakeup_interrupt
* Description  : This function is LIN Wakeup/INTP11 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
//void LIN_Wakeup_interrupt(void)
//{
//    LIN->LCUC=0x03;
//}

