/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    pga.c
* @brief   This file implements device driver for PGA module.
* @version 1.0.0
* @date    2022/2/25
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "BAT32G113.h"
#include "pga.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: PGA_Init
* @brief  This function initializes the Programmable Gain Amplifier module.
* @param  ch - specify the pga channel number
* @param  mode - GND selection of feedback resistance
* @param  vref - 1/2VDD, 1.45V 2.4V
* @param  gain - Programmable gain amplifier amplification factor selection
* @return None
***********************************************************************************************************************/
void PGA_Init(pga_channel_t ch, pga_mode_t mode, pga_vref_t vref, pga_gain_t gain)
{
    CGC->PER1 |= CGC_PER1_PGACMPEN_Msk;    /* enables input clock supply */

    PGA->PGA0CTL = 0x00U;     /* PGA operation stopped */
    PGA->PGA0CTL = ch | mode | gain;
    /* Set PGA0 pin */
    if (ch==PGA_CHANNEL_0){
        PGA0IN_PORT_SETTING();
        if(mode == PGA_PGAxGND_INPUT)
        {
            PGA0GND_PORT_SETTING();
        }
    }
    else if(ch==PGA_CHANNEL_1){
        PGA1IN_PORT_SETTING(); // use single mode
    }
    if (vref==HALF_VDD){
        //PGA->PGA0CTL |= 0x00; // dummy
    }
    else if(vref==INNER_1P45V){
        PGA->PGA0CTL |= 0x10;
        MISC->VREFCTL = 0x00;
    }
    else if(vref==INNER_2P4V){
        PGA->PGA0CTL |= 0x10;
        MISC->VREFCTL = 0x04;
    }
}

/***********************************************************************************************************************
* Function Name: PGA_Start
* @brief  This function starts the  programmable gain amplifier.
* @return None
***********************************************************************************************************************/
void PGA_Start(void)
{     
    PGA->PGA0CTL |= PGAEN;     /* PGA0 operation enabled */
}
/***********************************************************************************************************************
* Function Name: PGA_Stop
* @brief  This function stops the  programmable gain amplifier.
* @return None
***********************************************************************************************************************/
void PGA_Stop(void)
{
    PGA->PGA0CTL &= ~PGAEN;     /* PGA0 operation stopped */
}

/***********************************************************************************************************************
* Function Name: PGAOUT_Enable
* @brief  This function starts the  programmable gain amplifier.
* @return None
***********************************************************************************************************************/
void PGAOUT_Enable(void)
{     
    PGA->PGA0CTL |= PGA0EN;     /* PGA0OUT/P20 output enabled */
}
/***********************************************************************************************************************
* Function Name: PGAOUT_Disable
* @brief  This function stops the  programmable gain amplifier.
* @return None
***********************************************************************************************************************/
void PGAOUT_Disable(void)
{
    PGA->PGA0CTL &= ~PGA0EN;     /* PGA0OUT/P20 output disabled */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
