/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    spi.c
* @brief   This file implements device driver for SPI module.
* @version 1.0.0
* @date    2020/10/24
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "BAT32G113.h"
#include "userdefine.h"
#include "spi.h"
#include "dma.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma diag_suppress 550
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
volatile uint8_t *gp_spihs0_rx_address;       /* spi receive buffer address */
volatile uint16_t g_spihs0_rx_length;         /* spi receive data length */
volatile uint16_t g_spihs0_rx_count;          /* spi receive data count */
volatile uint8_t *gp_spihs0_tx_address;       /* spi send buffer address */
volatile uint16_t g_spihs0_tx_length;         /* spi send data length */
volatile uint16_t g_spihs0_tx_count;          /* spi send data count */

/***********************************************************************************************************************
* Function Name: SPIHS0_MasterInit
* @brief  This function restarts the SPI0 module.
* @param  mode
*             - SPI_MODE_0: CPOL = 0, CPHA = 0; i.e. CKP = 1, DAP = 1
*             - SPI_MODE_1: CPOL = 0, CPHA = 1; i.e. CKP = 1, DAP = 0
*             - SPI_MODE_2: CPOL = 1, CPHA = 0; i.e. CKP = 0, DAP = 1
*             - SPI_MODE_3: CPOL = 1, CPHA = 1; i.e. CKP = 0, DAP = 0
* @return None
***********************************************************************************************************************/
void SPIHS0_MasterInit(spi_mode_t mode)
{
    CGC->PER2 |= CGC_PER2_SPIHSEN_Msk;
#ifdef SPIHS0_WITH_DMA
    SPIHS0->SPIC = _0004_SPI_SCK_fCLK_4 | (mode & 0x03) << 3;    
#else
    SPIHS0->SPIC = _0006_SPI_SCK_fCLK_6 | (mode & 0x03) << 3;    
#endif
    SPIHS0->SPIM = _0040_SPI_RECEPTION_TRANSMISSION | _0000_SPI_MSB | _0000_SPI_TRANSFER_END | _0000_SPI_LENGTH_8;
    NSS0_PORT_SETTING();
    SCK0_PORT_SETTING();
    MOSI0_PORT_SETTING();
    MISO0_PORT_SETTING();
}

/***********************************************************************************************************************
* Function Name: SPIHS0_Start
* @brief  This function starts the SPI0 module operation.
* @param  None
* @return None
***********************************************************************************************************************/
void SPIHS0_Start(void)
{
    /* clear INTSPI interrupt flag */
    INTC_ClearPendingIRQ(SPIHS0_IRQn);
    NVIC_ClearPendingIRQ(SPIHS0_IRQn);
    /* enable INTSPI interrupt */
    INTC_EnableIRQ(SPIHS0_IRQn);
    NVIC_EnableIRQ(SPIHS0_IRQn);
    SPIHS0->SPIM |= _0080_SPI_START_TRG_ON; 
    /* SSn Active */
    NSS0_PORT_CLR();
}
/***********************************************************************************************************************
* Function Name: SPIHS0_Stop
* @brief  This function stops the SPI0 module operation.
* @param  None
* @return None
***********************************************************************************************************************/
void SPIHS0_Stop(void)
{
    /* SSn Inactive */
    NSS0_PORT_SET();
    SPIHS0->SPIM &= ~_0080_SPI_START_TRG_ON; 
    /* disable INTSPI interrupt */
    INTC_DisableIRQ(SPIHS0_IRQn);
    NVIC_DisableIRQ(SPIHS0_IRQn);
    /* clear INTSPI interrupt flag */
    INTC_ClearPendingIRQ(SPIHS0_IRQn);
    NVIC_ClearPendingIRQ(SPIHS0_IRQn);
}
/***********************************************************************************************************************
* Function Name: SPIHS0_MasterSend
* @brief  This function sends SPI data.
* @param  tx_buf - transfer buffer pointer
* @param  tx_num - buffer size
* @return status - MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS SPIHS0_MasterSend(uint8_t *const tx_buf, uint16_t tx_num)
{
    MD_STATUS status = MD_OK;

    if (tx_num < 1U)
    {
        status = MD_ERROR;
    }
    else
    {
        SPIHS0->SPIM |= _0040_SPI_RECEPTION_TRANSMISSION | _0008_SPI_BUFFER_EMPTY; /* transmission mode */
#ifdef SPIHS0_WITH_DMA
        /* write transfer data with DMA */
        DMAVEC->VEC[DMA_VECTOR_SPIHS0] = CTRL_DATA_SPIHS0;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACR = (0 << CTRL_DMACR_SZ_Pos) | (0 << CTRL_DMACR_CHNE_Pos) |
                                               (0 << CTRL_DMACR_DAMOD_Pos) | (1 << CTRL_DMACR_SAMOD_Pos) |
                                               (0 << CTRL_DMACR_MODE_Pos);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMBLS = 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACT = tx_num - 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMRLD = tx_num - 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMSAR = (uint32_t)(tx_buf + 1);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMDAR = (uint32_t)&SPIHS0->SDRO;

        /* init DMA registers */
        CGC->PER1 |= CGC_PER1_DMAEN_Msk;
        DMA->DMABAR = DMAVEC_BASE;
        DMA->DMAEN2 |= (1 << DMA_VECTOR_SPIHS0 % 8);
#endif
#ifdef SPIHS0_WITH_DMA
        g_spihs0_tx_count = 1;         /* send data count */
        gp_spihs0_tx_address = tx_buf; /* send buffer pointer */
#else
        g_spihs0_tx_count = tx_num;    /* send data count */
        gp_spihs0_tx_address = tx_buf; /* send buffer pointer */
#endif
        gp_spihs0_tx_address++;
        g_spihs0_tx_count--;
        SPIHS0_Start();
        SPIHS0->SDRO = tx_buf[0]; /* started by writing data to SDRO */
    }

    return (status);
}

/***********************************************************************************************************************
* Function Name: SPIHS0_MasterReceive
* @brief  This function receives SPI0 data.
* @param  rx_buf - receive buffer pointer
* @param  rx_num - buffer size
* @return status - MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS SPIHS0_MasterReceive(uint8_t *const rx_buf, uint16_t rx_num)
{
    MD_STATUS status = MD_OK;
    uint32_t dummy_sdr;

    if (rx_num < 1U)
    {
        status = MD_ERROR;
    }
    else
    {
        SPIHS0->SPIM &= ~(_0040_SPI_RECEPTION_TRANSMISSION | _0008_SPI_BUFFER_EMPTY); /* reception mode */
#ifdef SPIHS0_WITH_DMA
        /* read receive data with DMA */
        DMAVEC->VEC[DMA_VECTOR_SPIHS0] = CTRL_DATA_SPIHS0;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACR = (0 << CTRL_DMACR_SZ_Pos) | (0 << CTRL_DMACR_CHNE_Pos) |
                                               (1 << CTRL_DMACR_DAMOD_Pos) | (0 << CTRL_DMACR_SAMOD_Pos) |
                                               (0 << CTRL_DMACR_MODE_Pos);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMBLS = 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACT = rx_num;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMRLD = rx_num;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMSAR = (uint32_t)&SPIHS0->SDRI;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMDAR = (uint32_t)rx_buf;

        /* init DMA registers */
        CGC->PER1 |= CGC_PER1_DMAEN_Msk;
        DMA->DMABAR = DMAVEC_BASE;
        DMA->DMAEN2 |= (1 << DMA_VECTOR_SPIHS0 % 8);
#endif
#ifdef SPIHS0_WITH_DMA
        g_spihs0_rx_count = 0;                      /* receive data count */
        gp_spihs0_rx_address = rx_buf + rx_num ;    /* receive buffer pointer */
#else
        g_spihs0_rx_count = rx_num;    /* receive data count */
        gp_spihs0_rx_address = rx_buf; /* receive buffer pointer */
#endif

        SPIHS0_Start();
        dummy_sdr = SPIHS0->SDRI;
    }

    return (status);
}
/***********************************************************************************************************************
* Function Name: SPIHS0_SlaveInit
* @brief  This function restarts the SPI module.
* @param  mode
*             - SPI_MODE_0: CPOL = 0, CPHA = 0; i.e. CKP = 1, DAP = 1
*             - SPI_MODE_1: CPOL = 0, CPHA = 1; i.e. CKP = 1, DAP = 0
*             - SPI_MODE_2: CPOL = 1, CPHA = 0; i.e. CKP = 0, DAP = 1
*             - SPI_MODE_3: CPOL = 1, CPHA = 1; i.e. CKP = 0, DAP = 0
* @return None
***********************************************************************************************************************/
void SPIHS0_SlaveInit(spi_mode_t mode)
{
    CGC->PER2 |= CGC_PER2_SPIHSEN_Msk;
    SPIHS0->SPIC = _0007_SPI_SLAVE_MODE | (mode & 0x03) << 3;    
    SPIHS0->SPIM = _0040_SPI_RECEPTION_TRANSMISSION | _0020_SPI_NSS_ENABLE | 
	                  _0000_SPI_MSB | _0000_SPI_TRANSFER_END | _0000_SPI_LENGTH_8;
    NSS0_PORT_SETTING();
    SCK0_PORT_SETTING();
    MOSI0_PORT_SETTING();
    MISO0_PORT_SETTING();
}
/***********************************************************************************************************************
* Function Name: SPIHS0_SlaveSend
* @brief  This function sends SPI data.
* @param  tx_buf - transfer buffer pointer
* @param  tx_num - buffer size
* @return status - MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS SPIHS0_SlaveSend(uint8_t *const tx_buf, uint16_t tx_num)
{
    MD_STATUS status = MD_OK;

    if (tx_num < 1U)
    {
        status = MD_ERROR;
    }
    else
    {
        SPIHS0->SPIM |= _0040_SPI_RECEPTION_TRANSMISSION | _0000_SPI_TRANSFER_END | _0020_SPI_NSS_ENABLE; /* transmission mode */
#ifdef SPIHS0_WITH_DMA
        /* write transfer data with DMA */
        DMAVEC->VEC[DMA_VECTOR_SPIHS0] = CTRL_DATA_SPIHS0;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACR = (0 << CTRL_DMACR_SZ_Pos) | (0 << CTRL_DMACR_CHNE_Pos) |
                                               (0 << CTRL_DMACR_DAMOD_Pos) | (1 << CTRL_DMACR_SAMOD_Pos) |
                                               (0 << CTRL_DMACR_MODE_Pos);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMBLS = 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACT = tx_num - 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMRLD = tx_num - 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMSAR = (uint32_t)(tx_buf + 1);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMDAR = (uint32_t)&SPIHS0->SDRO;

        /* init DMA registers */
        CGC->PER1 |= CGC_PER1_DMAEN_Msk;
        DMA->DMABAR = DMAVEC_BASE;
        DMA->DMAEN2 |= (1 << DMA_VECTOR_SPIHS0 % 8);
#endif
#ifdef SPIHS0_WITH_DMA
        g_spihs0_tx_count = 1;         /* send data count */
        gp_spihs0_tx_address = tx_buf; /* send buffer pointer */
#else
        g_spihs0_tx_count = tx_num;    /* send data count */
        gp_spihs0_tx_address = tx_buf; /* send buffer pointer */
#endif
        gp_spihs0_tx_address++;
        g_spihs0_tx_count--;
        SPIHS0_Start();
        SPIHS0->SDRO = tx_buf[0]; /* started by writing data to SDRO */
    }

    return (status);
}

/***********************************************************************************************************************
* Function Name: SPIHS0_SlaveReceive
* @brief  This function receives SPI data.
* @param  rx_buf - receive buffer pointer
* @param  rx_num - buffer size
* @return status - MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS SPIHS0_SlaveReceive(uint8_t *const rx_buf, uint16_t rx_num)
{
    MD_STATUS status = MD_OK;
    uint32_t dummy_sdr;

    if (rx_num < 1U)
    {
        status = MD_ERROR;
    }
    else
    {
        SPIHS0->SPIM &= ~(_0040_SPI_RECEPTION_TRANSMISSION | _0008_SPI_BUFFER_EMPTY); /* reception mode */
        SPIHS0->SPIM |= _0020_SPI_NSS_ENABLE;
#ifdef SPIHS0_WITH_DMA
        /* read receive data with DMA */
        DMAVEC->VEC[DMA_VECTOR_SPIHS0] = CTRL_DATA_SPIHS0;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACR = (0 << CTRL_DMACR_SZ_Pos) | (0 << CTRL_DMACR_CHNE_Pos) |
                                               (1 << CTRL_DMACR_DAMOD_Pos) | (0 << CTRL_DMACR_SAMOD_Pos) |
                                               (0 << CTRL_DMACR_MODE_Pos);
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMBLS = 1;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMACT = rx_num;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMRLD = rx_num;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMSAR = (uint32_t)&SPIHS0->SDRI;
        DMAVEC->CTRL[CTRL_DATA_SPIHS0].DMDAR = (uint32_t)rx_buf;

        /* init DMA registers */
        CGC->PER1 |= CGC_PER1_DMAEN_Msk;
        DMA->DMABAR = DMAVEC_BASE;
        DMA->DMAEN2 |= (1 << DMA_VECTOR_SPIHS0 % 8);
#endif
#ifdef SPIHS0_WITH_DMA
        g_spihs0_rx_count = 0;                      /* receive data count */
        gp_spihs0_rx_address = rx_buf + rx_num; /* receive buffer pointer */
#else
        g_spihs0_rx_count = rx_num;    /* receive data count */
        gp_spihs0_rx_address = rx_buf; /* receive buffer pointer */
#endif

        SPIHS0_Start();
        dummy_sdr = SPIHS0->SDRI;
    }

    return (status);
}
/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
