/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    spi_user.c
* @brief   This file implements device driver for SPI module.
* @version 1.0.0
* @date    2020/10/24
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "BAT32G113.h"
#include "spi.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
void IRQ09_Handler(void) __attribute__((alias("spihs0_interrupt")));
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: spihs0_interrupt
* @brief  SPI0 interrupt service routine
* @param  None
* @return None
***********************************************************************************************************************/
void spihs0_interrupt(void)
{
    volatile uint8_t sio_dummy;

    if (g_spihs0_tx_count > 0U || g_spihs0_rx_count > 1U)
    {
        /* reveive data */
        if (0U != gp_spihs0_rx_address)
        {
            *gp_spihs0_rx_address = SPIHS0->SDRI;
            gp_spihs0_rx_address++;
            g_spihs0_rx_count--;
        }
        else
        {
            sio_dummy = SPIHS0->SDRI;
        }

        /* send data */
        if (0U != gp_spihs0_tx_address)
        {
            SPIHS0->SDRO = *gp_spihs0_tx_address;
            gp_spihs0_tx_address++;
            g_spihs0_tx_count--;
        }
        else
        {
            SPIHS0->SDRO = 0xFFU;
        }

    }
    else 
    {
        /* CPU receive the last data */
        if (1U == g_spihs0_rx_count)
        {
            SPIHS0->SPIM &= ~_0080_SPI_START_TRG_ON;
            if (0U != gp_spihs0_rx_address)
            {
                *gp_spihs0_rx_address = SPIHS0->SDRI;
                 g_spihs0_rx_count--;
            }
            else
            {
                sio_dummy = SPIHS0->SDRI;
            }
        }
        if (0U != gp_spihs0_rx_address)
        {
            spihs0_callback_receiveend();    /* complete receive */
        }
        /* wait the last data sent for DMA transfer */
        if (0U != gp_spihs0_tx_address)
        {
            while(SPIHS0->SPIS & _0001_SPI_UNDER_EXECUTE) 
            {
                ;
            }
            spihs0_callback_sendend();    /* complete send */
        }
    }
    INTC_ClearPendingIRQ(SPIHS0_IRQn);
}
/***********************************************************************************************************************
* Function Name: spihs0_callback_receiveend
* @brief  This function is a callback function when spi finishes reception.
* @param  None
* @return None
***********************************************************************************************************************/
static void spihs0_callback_receiveend(void)
{
    /* Start user code. Do not edit comment generated here */
    SPIHS0_Stop();
    gp_spihs0_rx_address = 0;    /* reinitialize global variable */
    /* End user code. Do not edit comment generated here */
}
/***********************************************************************************************************************
* Function Name: spihs0_callback_sendend
* @brief  This function is a callback function when spi finishes transmission.
* @param  None
* @return None
***********************************************************************************************************************/
static void spihs0_callback_sendend(void)
{
    /* Start user code. Do not edit comment generated here */
    SPIHS0_Stop();
    gp_spihs0_tx_address = 0;    /* reinitialize global variable */
    /* End user code. Do not edit comment generated here */
}
/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
