/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G113
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2022/2/25
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include "BAT32G113.h"
#include "userdefine.h"
#include "sci.h"
#include "dac.h"
#include "cmp.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
volatile uint32_t g_ticks;
uint8_t set_value[8];
uint8_t get_value[8];

void delayMS(uint32_t n)
{
		g_ticks = n;
		while(g_ticks);
}

int main()
{
	MD_STATUS status;
	uint32_t msCnt; 	// count value of 1ms
	uint32_t i;

//-----------------------------------------------------------------------
// Systick setting 
//-----------------------------------------------------------------------
	g_ticks = 1000;
	SystemCoreClockUpdate();
	msCnt = SystemCoreClock / 1000;
	SysTick_Config(msCnt);  
	
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART0_Init(SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);
	}
	
	printf("Hello, UART\n");
#endif

//-----------------------------------------------------------------------
// P22: DAC channel 0 output --> CMP channel 0 positive side input
//-----------------------------------------------------------------------
	printf("DAC(ch0) --> CMP(ch0) nSide\n");	
	DAC_Init(DAC_CHANNEL_0);
	DAC_Start(DAC_CHANNEL_0); 
    DAC->DACS0 = 0xFFF;
	CMP_Init(CMP_CHANNEL_0, CMP_PSIDE_VCIN0, CMP_NSIDE_DAC);
    CMP_Hy_Setting(CMP_CHANNEL_0, CMP_HYEDGE_BOTH, CMP_HYVAL_40mv);
	CMP_Filter_Setting(CMP_CHANNEL_0, CMP_FILTER_FCLK_32, CMP_EDGE_RISING);
	CMP_Start(CMP_CHANNEL_0);

    for(i=256; i>0; i--) 
    {
        DAC->DACS0 = i*16-1;
        delayMS(1);
        if(g_Cmp0IntTaken) 
        {
            printf("nSide Set Value = 0x%04X, ", DAC->DACS0);
            printf("pSide Set Value = VCIN0(input 2.5V), ");        // VCIN0/P22
            printf("C0MON = %d, ", (CMP->COMPMDR & CMP_COMPMDR_C0MON_Msk) >> 3);
            printf("IntTaken = %d\n", g_Cmp0IntTaken);	
            break;
        }
    }

	CMP_Stop(CMP_CHANNEL_0);	
	DAC_Stop(DAC_CHANNEL_0);

    while(1){
    }
}
/***********************************************************************************************************************
* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

/***********************************************************************************************************************
* Function Name: HardFault_Handler
* Description  : Hard Fault handler to report stacked register values
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
// Hard Fault handler in C, with stack frame location and LR value
// extracted from the assembly wrapper as input parameters
void hard_fault_handler_c(unsigned int * hardfault_args, unsigned lr_value)
{
	unsigned int stacked_r0;
	unsigned int stacked_r1;
	unsigned int stacked_r2;
	unsigned int stacked_r3;
	unsigned int stacked_r12;
	unsigned int stacked_lr;
	unsigned int stacked_pc;
	unsigned int stacked_psr;
	
	stacked_r0 = ((unsigned long) hardfault_args[0]);
	stacked_r1 = ((unsigned long) hardfault_args[1]);
	stacked_r2 = ((unsigned long) hardfault_args[2]);
	stacked_r3 = ((unsigned long) hardfault_args[3]);
	stacked_r12 = ((unsigned long) hardfault_args[4]);
	stacked_lr = ((unsigned long) hardfault_args[5]);
	stacked_pc = ((unsigned long) hardfault_args[6]);
	stacked_psr = ((unsigned long) hardfault_args[7]);
	
	printf ("[Hard fault handler]\r\n");
	printf ("R0 = %x\r\n", stacked_r0);
	printf ("R1 = %x\r\n", stacked_r1);
	printf ("R2 = %x\r\n", stacked_r2);
	printf ("R3 = %x\r\n", stacked_r3);
	printf ("R12 = %x\r\n", stacked_r12);
	printf ("Stacked LR = %x\r\n", stacked_lr);
	printf ("Stacked PC = %x\r\n", stacked_pc);
	printf ("Stacked PSR = %x\r\n", stacked_psr);
	printf ("Current LR = %x\r\n", lr_value);
	
	while(1); // endless loop
}

/***********************************************************************************************************************
* Function Name: HardFault_Handler
* Description  : Assembly wrapper using Embedded Assembler in Keil MDK
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
// Hard Fault handler wrapper in assembly
// It extracts the location of stack frame and passes it to handler
// in C as a pointer. We also extract the LR value as second
// parameter.
#if defined (__CC_ARM)
__asm void HardFault_Handler(void)
{
		MOVS 	r0, #4
		MOV 	r1, LR
		TST 	r0, r1
		BEQ 	stacking_used_MSP
		MRS 	R0, PSP ; first parameter - stacking was using PSP
		B 		get_LR_and_branch
stacking_used_MSP
		MRS 	R0, MSP ; first parameter - stacking was using MSP
get_LR_and_branch
		MOV 	R1, LR ; second parameter is LR current value
		LDR 	R2,=__cpp(hard_fault_handler_c)
		BX 		R2
}

#elif defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6100100)
void HardFault_Handler(void)
{
	__ASM volatile(
		"MOVS 	r0, #4\n"
		"MOV 	r1, LR\n"
		"TST 	r0, r1\n"
		"BEQ 	stacking_used_MSP\n"
		"MRS 	R0, PSP\n"
		"B 		get_LR_and_branch\n"
"stacking_used_MSP:\n"
		"MRS 	R0, MSP\n"
"get_LR_and_branch:\n"
		"MOV 	R1, LR\n"
		"LDR 	R2,= hard_fault_handler_c\n"
		"BX 		R2\n"
		);
}
#endif

